@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2022 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -40,11 +40,20 @@ our @EXPORT = qw(
     nr_readlink
     nr_stat
     nr_fork_pipe
+    nr_fork_pipe_stderr
     nr_fork_pipew
     nr_fork_pipe2
+    nr_get_cgroup
+    nr_is_systemd_manager
+    nr_get_ns
+    nr_get_pid_ns
 );
 
-my %ptable = map {$_->pid => $_} @{ new Proc::ProcessTable(enable_ttys => 1)->table };
+my %ptable;
+{
+    local $SIG{__WARN__} = sub {};
+    %ptable = map {$_->pid => $_} @{ new Proc::ProcessTable(enable_ttys => 1)->table };
+}
 
 sub nr_ptable() {
     return \%ptable;
@@ -56,6 +65,51 @@ sub nr_ptable_pid($) {
     return $ptable{$pid};
 }
 
+sub nr_get_cgroup($) {
+    my $pid = shift;
+
+    # get unit name from /proc/<pid>/cgroup
+    if(open(HCGROUP, qq(/proc/$pid/cgroup))) {
+	my ($cgroup) = map {
+	    chomp;
+	    my ($id, $type, $value) = split(/:/);
+	    if(($id == 0 && $type eq "") || ($type eq q(name=systemd))) {
+		($value);
+	    } else {
+		();
+	    }
+	} <HCGROUP>;
+	close(HCGROUP);
+
+	return $cgroup;
+    }
+
+    return undef;
+}
+
+sub nr_is_systemd_manager($) {
+    my $pid = shift;
+    my $cgroup = nr_get_cgroup($pid);
+
+    return $cgroup =~ m@(^|/user\@(\d+)\.service)/init\.scope$@ if($cgroup);
+    return 0;
+}
+
+sub nr_get_ns($$) {
+    my $pid = shift;
+    my $ns = shift;
+
+    my $stat = nr_stat(qq(/proc/$pid/ns/$ns));
+
+    return $stat->{ino} if($stat);
+    return undef;
+}
+
+sub nr_get_pid_ns($) {
+    my $pid = shift;
+    return nr_get_ns($pid, 'pid');
+}
+
 sub nr_parse_cmd($) {
     my $pid = shift;
 
@@ -141,6 +195,25 @@ sub nr_fork_pipe($@) {
     \*HPIPE
 }
 
+sub nr_fork_pipe_stderr {
+    my $debug = shift;
+
+    my $pid = open(HPIPE, '-|');
+    defined($pid) || die "Can't fork: $!\n";
+
+    if($pid == 0) {
+	open(STDERR, '>&', STDOUT) || die "Can't dup stderr: $!\n";
+	close(STDIN);
+
+	undef $ENV{LANG};
+
+	exec(@_);
+	exit;
+    }
+
+    \*HPIPE
+}
+
 sub nr_fork_pipew($@) {
     my $debug = shift;
 
